// To parse this JSON data, do
//
//     final getLeaderboardOnRewards = getLeaderboardOnRewardsFromJson(jsonString);

import 'dart:convert';
import 'dart:developer';
import 'package:dio/dio.dart' as dio;

GetNudgeLeaderboard getLeaderboardOnRewardsFromJson(String str) =>
    GetNudgeLeaderboard.fromJson(json.decode(str));

String getLeaderboardOnRewardsToJson(GetNudgeLeaderboard data) =>
    json.encode(data.toJson());

class GetNudgeLeaderboard {
  String status;
  Data data;

  GetNudgeLeaderboard({
    required this.status,
    required this.data,
  });

  factory GetNudgeLeaderboard.fromJson(Map<String, dynamic> json) =>
      GetNudgeLeaderboard(
        status: json["status"],
        data: Data.fromJson(json["data"]),
      );

  Map<String, dynamic> toJson() => {
        "status": status,
        "data": data.toJson(),
      };

  append(GetNudgeLeaderboard leaderboard2) {
    data.append(leaderboard2.data);
  }
}

Future<GetNudgeLeaderboard> getNudgeLeaderboard(
  String? authToken,
  String apiUrl,
) async {
  var headers = {'Authorization': authToken};
  var request = dio.Dio();
  var response = await request.request(
    apiUrl,
    options: dio.Options(
      method: 'GET',
      headers: headers,
    ),
  );

  if (response.statusCode == 200) {
    if (response.data != null && response.data is Map<String, dynamic>) {
      try {
        GetNudgeLeaderboard.fromJson(response.data);
      } catch (error) {
        print(error);
      }
      return GetNudgeLeaderboard.fromJson(response.data);
    } else {
      // Handle the case where response.data is null or not a valid JSON object.
      throw Exception('Invalid JSON data received');
    }
  } else {
    throw Exception('Failed to fetch leaderboard data');
  }
}

class Data {
  User? user;
  int? rank;
  List<User> users;

  Data({
    this.user,
    this.rank,
    required this.users,
  });

  factory Data.fromJson(Map<String, dynamic> json) => Data(
        user: json["user"] != null ? User.fromJson(json["user"]) : null,
        rank: json["rank"],
        users: json["users"] != null
            ? List<User>.from(json["users"].map((x) => User.fromJson(x)))
            : [],
      );

  Map<String, dynamic> toJson() => {
        "user": user?.toJson(),
        "rank": rank,
        "users": List<dynamic>.from(users.map((x) => x.toJson())),
      };

  Data append(Data data2) {
    users.addAll(data2.users);
    return this;
  }
}

/// Updated user model
/// "_id": "65f93d97c9fba168d446acb7",
//  "totalQuantity": 194,
//  "userName": "User",
//  "rank": 1,
//  "externalId": "B9YJWPBR",
//  "createdAt": "2024-03-19T07:24:07.661Z",
//  "updatedAt": "2024-04-30T15:40:10.791Z",
//  "score": 1940
class User {
  String id;

  // UserId? userId;
  // String? rewardId;
  // String? compositeId;
  // int score;
  // DateTime createdAt;
  // DateTime updatedAt;
  int totalQuantity;
  String userName;
  int rank;
  String externalId;
  DateTime createdAt;
  DateTime updatedAt;
  int score;
  String? photoUrl;

  User(
      {required this.id,
      required this.totalQuantity,
      required this.userName,
      required this.rank,
      required this.externalId,
      required this.score,
      required this.createdAt,
      required this.updatedAt,
      this.photoUrl});

  factory User.fromJson(Map<String, dynamic> json) {
    return User(
      id: json["_id"],
      totalQuantity: json["totalQuantity"],
      userName: json["userName"],
      rank: json["rank"],
      score: json["score"],
      photoUrl: json["photoUrl"],
      externalId: json["externalId"],
      createdAt: DateTime.parse(json["createdAt"]),
      updatedAt: DateTime.parse(json["updatedAt"]),
    );
  }

  Map<String, dynamic> toJson() => {
        "_id": id,
        "totalQuantity": totalQuantity,
        "userName": userName,
        "rank": rank,
        "score": score,
        "photoUrl": photoUrl,
        "createdAt": createdAt.toIso8601String(),
        "updatedAt": updatedAt.toIso8601String(),
      };
}

class UserId {
  String id;
  String? name;
  String? email;
  String externalId;
  String? photoUrl;

  UserId({
    required this.id,
    this.name,
    this.email,
    required this.externalId,
    this.photoUrl,
  });

  factory UserId.fromJson(Map<String, dynamic> json) => UserId(
        id: json["_id"],
        name: json["name"],
        email: json["email"],
        externalId: json["externalId"],
        photoUrl: json["photoUrl"],
      );

  Map<String, dynamic> toJson() => {
        "_id": id,
        "name": name,
        "email": email,
        "externalId": externalId,
        "photoUrl": photoUrl,
      };
}
