import 'dart:developer';

import 'package:nudge_leaderboard_ui/features/leaderboard/data/datasources/createuser.dart';
import 'package:nudge_leaderboard_ui/features/leaderboard/data/models/create_user.dart';
import 'package:nudge_leaderboard_ui/features/leaderboard/presentation/widgets/leaderboard_nodata.dart';
import 'package:nudge_leaderboard_ui/features/leaderboard/presentation/widgets/leaderboard_scoredata.dart';

import '../../../../main.dart';
import '../../data/models/get_all_leaderboard_rewards.dart';
import '../../data/models/get_nudge_leaderboard.dart';
import '../widgets/appbar/leaderboard_appbar.dart';
import 'package:flutter/material.dart';

class LeaderboardScreen extends StatefulWidget {
  final String token;
  final String? rewardID;
  final String? externalID;
  final Map<String, dynamic>? props;

  const LeaderboardScreen({
    super.key,
    required this.token,
    this.rewardID,
    required this.externalID,
    this.props,
  });

  @override
  State<LeaderboardScreen> createState() =>
      // ignore: no_logic_in_create_state
      _LeaderboardScreenState(rewardID: rewardID);
}

class _LeaderboardScreenState extends State<LeaderboardScreen> {
  String? rewardID;
  bool isLoading = false;
  int currentPageNo = 1;
  int currentPageNoTimeBased = 2;
  int currentItemCount = 10;
  final int increment = 10;
  GetNudgeLeaderboard? nudgeLeaderboardData, nudgeLeaderboardRankers;
  GetAllLeaderboardRewards? allLeaderboardRewardsData;
  CreateUser? userData;

  bool isDataFetched = false;

  DateTime now = DateTime.now();
  late int startDate;
  late int endDate;

  _LeaderboardScreenState({
    this.rewardID,
  });

  @override
  void initState() {
    super.initState();

    startDate = DateTime(now.year, now.month, 1).millisecondsSinceEpoch;
    endDate = DateTime.now().millisecondsSinceEpoch;

    // log("startDate: $startDate, endDate: $endDate", name: "LeaderboardScreen");
    //
    fetchRewardsData().then((_) {
      if (rewardID?.isEmpty == true) {
      } else {}
    });
  }

  Future<void> updateDateRange(int startDate, int endDate, bool refresh) async {
    try {
      this.startDate = startDate;
      this.endDate = endDate;

      if (refresh) {
        currentItemCount = 10;
        currentPageNo = 1;

        GetNudgeLeaderboard fetchedRewardsLeaderboard =
            await getNudgeLeaderboard(
          userData?.data.token,
          '$nudgeUrl/leaderboard/lb/get/reward/$rewardID?pageNo=$currentPageNo&noPerPage=$currentItemCount',
        );

        // log(
        //   "size: ${fetchedRewardsLeaderboard.data.users.length}",
        //   name: 'update load',
        // );

        setState(() {
          nudgeLeaderboardData = fetchedRewardsLeaderboard;
          isDataFetched = true;
        });

        return;
      }

      if (userData == null) {
        return;
      }

      // nudgeLeaderboardData = null;

      // log(
      //   "itemCount: $currentItemCount, pageNo: $currentPageNoTimeBased, nudgeLeaderboard: ${nudgeLeaderboardData?.data.users.length}",
      //   name: 'load more',
      // );

      GetNudgeLeaderboard fetchedRewardsLeaderboard = await getNudgeLeaderboard(
        userData?.data.token,
        '$nudgeUrl/leaderboard/lb/get/reward/$rewardID?filter={"createdAt":{"\$gte":$startDate,"\$lte":$endDate}}&pageNo=$currentPageNoTimeBased&noPerPage=$currentItemCount',
      );

      if (nudgeLeaderboardData != null) {
        setState(() {
          nudgeLeaderboardData!.append(fetchedRewardsLeaderboard);
        });
      } else {
        setState(() {
          nudgeLeaderboardData = fetchedRewardsLeaderboard;
          isDataFetched = true;
        });
      }

      // log(
      //   "itemCount: $currentItemCount, pageNo: $currentPageNoTimeBased, nudgeLeaderboard: ${nudgeLeaderboardData?.data.users.length}",
      //   name: 'load more',
      // );

      // await fetchTimeBasedLeaderboardOnScore();
      // await fetchLeaderboardOnReward(rewardID);}
    } catch (error) {
      debugPrint("Error: $error");
    }
  }

  Future<void> fetchRewardsData() async {
    try {
      CreateUser createUser = await createuserrequest(
        widget.token,
        widget.externalID,
        props: widget.props,
      );

      setState(() {
        userData = createUser;
      });
    } catch (error) {
      debugPrint('Error: $error');
      setState(() {
        nudgeLeaderboardData = null;
      });
      return;
    }
  }

  // Future<void> fetchLeaderboard() async {
  //   try {
  //     GetNudgeLeaderboard fetchedRewardsLeaderboard = await getNudgeLeaderboard(
  //       userData?.data.token,
  //       '$nudgeUrl/users/u/get/leaderboard/score',
  //     );
  //     setState(() {
  //       nudgeLeaderboardData = fetchedRewardsLeaderboard;
  //     });
  //   } catch (error) {
  //     debugPrint('Error: $error');
  //   }
  // }

  Future<void> fetchLeaderboardOnReward(String? rewardID) async {
    try {
      if (userData == null) {
        return;
      }

      GetNudgeLeaderboard fetchedRewardsLeaderboard = await getNudgeLeaderboard(
        userData?.data.token,
        '$nudgeUrl/leaderboard/lb/get/reward/$rewardID?pageNo=$currentPageNo&noPerPage=$currentItemCount',
      );

      if (nudgeLeaderboardData != null) {
        setState(() {
          nudgeLeaderboardData!.append(fetchedRewardsLeaderboard);
        });
      } else {
        setState(() {
          nudgeLeaderboardRankers = fetchedRewardsLeaderboard;
          nudgeLeaderboardData = fetchedRewardsLeaderboard;

          isDataFetched = true;
        });
      }
    } catch (error) {
      debugPrint('Fetch Reward Leaderboard Error: $error');
    }
  }

  Future<void> fetchTimeBasedLeaderboardOnScore() async {
    try {
      GetNudgeLeaderboard fetchedRewardsLeaderboard = await getNudgeLeaderboard(
        userData?.data.token,
        '$nudgeUrl/leaderboard/lb/get/campaign/$rewardID&pageNo=$currentPageNo&noPerPage=$currentItemCount',
      );

      setState(() {
        nudgeLeaderboardData = fetchedRewardsLeaderboard;
        isDataFetched = true;
      });
    } catch (error) {
      debugPrint('Error: $error');
      setState(() {
        nudgeLeaderboardData = null;
      });
    }
  }

  Future<void> loadMoreData(LeaderBoardType leaderBoardType) async {
    if (isLoading) {
      return;
    }
    setState(() {
      isLoading = true;
      // currentPageNo = currentPageNo + 1;
      if (leaderBoardType == LeaderBoardType.timeBased) {
        currentPageNoTimeBased = currentPageNoTimeBased + 1;
      } else {
        currentPageNo = currentPageNo + 1;
      }
    });

    // log(
    //   "currentPageNo: $currentPageNo, currentItemCount: $currentItemCount, currentPageNoTimeBased: $currentPageNoTimeBased",
    //   name: 'load more',
    // );

    // log("dates $startDate, $endDate", name: 'load more');

    if (leaderBoardType == LeaderBoardType.timeBased) {
      await updateDateRange(startDate, endDate, false);
      // log("fetching time based leaderboard", name: 'load more');
    } else {
      await fetchLeaderboardOnReward(rewardID);
      // log("fetching reward based leaderboard", name: 'load more');
    }

    if (currentItemCount + increment <=
        (nudgeLeaderboardData?.data.users.length ?? 0)) {
      currentItemCount += increment;
    } else {
      currentItemCount = (nudgeLeaderboardData?.data.users.length ?? 0);
    }
    setState(() {
      isLoading = false;
    });
  }

  @override
  Widget build(BuildContext context) {
    // if (nudgeLeaderboardData == null) {
    //   return Scaffold(
    //     backgroundColor: const Color.fromRGBO(49, 162, 215, 1),
    //     appBar: const LeaderboardAppBar(),
    //     body: Center(
    //       child: Column(
    //         mainAxisAlignment: MainAxisAlignment.center,
    //         children: [
    //           _buildSubtitle("Loading..."),
    //           const SizedBox(height: 10),
    //           const CircularProgressIndicator(
    //             strokeWidth: 6,
    //             color: Colors.white,
    //           ),
    //         ],
    //       ),
    //     ),
    //   );
    // } else if (nudgeLeaderboardData!.data.users.isEmpty) {
    //   return LeaderboardNoData();
    // } else if (noData) {
    //   return LeaderboardNoData();
    // } else {
    //   return LeaderboardDataScore(
    //     nudgeLeaderboardData: nudgeLeaderboardData,
    //     currentItemCount: currentItemCount,
    //     currentPageNo: currentPageNo,
    //     loadMoreData: loadMoreData,
    //     refreshDataOnDateChanged: updateDateRange,
    //   );
    // }

    return FutureBuilder(
      future: isDataFetched
          ? null
          : (rewardID != null
              ? fetchLeaderboardOnReward(rewardID)
              : fetchTimeBasedLeaderboardOnScore()),
      builder: (context, snapshot) {
        if (snapshot.connectionState == ConnectionState.waiting) {
          return Scaffold(
            backgroundColor: const Color.fromRGBO(49, 162, 215, 1),
            appBar: const LeaderboardAppBar(),
            body: Center(
              child: Column(
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  _buildSubtitle("Loading..."),
                  const SizedBox(height: 10),
                  const CircularProgressIndicator(
                    strokeWidth: 6,
                    color: Colors.white,
                  ),
                ],
              ),
            ),
          );
        } else if (snapshot.hasError) {
          return LeaderboardNoData();
        } else {
          return LeaderboardDataScore(
            nudgeLeaderboardRankers: nudgeLeaderboardRankers,
            nudgeLeaderboardData: nudgeLeaderboardData,
            currentItemCount: currentItemCount,
            currentPageNo: currentPageNo,
            loadMoreData: loadMoreData,
            refreshDataOnDateChanged: updateDateRange,
            startDate: startDate,
            endDate: endDate,
          );
        }
      },
    );
  }
}

Widget _buildSubtitle(String title) {
  return Container(
    padding: const EdgeInsets.only(
      top: 24,
      bottom: 16,
    ),
    child: Text(
      title,
      style: const TextStyle(
        fontSize: 20,
        color: Colors.white,
      ),
    ),
  );
}
