import 'dart:developer';

import 'package:flutter/material.dart';

class LeaderboardAppBar extends StatefulWidget implements PreferredSizeWidget {
  final Function(
    int,
    int,
    bool,
  )? refreshDataOnDateChanged;
  final int? startDate;
  final int? endDate;

  const LeaderboardAppBar({
    super.key,
    this.refreshDataOnDateChanged,
    this.startDate,
    this.endDate,
  });

  @override
  State<LeaderboardAppBar> createState() => _LeaderboardAppBarState();

  @override
  Size get preferredSize => const Size.fromHeight(kToolbarHeight);
}

class _LeaderboardAppBarState extends State<LeaderboardAppBar> {
  bool isDataFetched = false;

  @override
  Widget build(BuildContext context) {
    return AppBar(
      backgroundColor: const Color.fromRGBO(49, 162, 215, 1),
      actions: [
        widget.refreshDataOnDateChanged != null
            ? IconButton(
                icon:
                    const Icon(Icons.date_range_outlined, color: Colors.white),
                onPressed: () {
                  DateTimeRange? dateRange;
                  DateTimeRange? initialDateRange;

                  DateTime now = DateTime.now();
                  DateTime firstDayOfMonth = DateTime(now.year, now.month, 1);

                  if (widget.startDate !=
                          firstDayOfMonth.millisecondsSinceEpoch ||
                      widget.endDate != now.millisecondsSinceEpoch) {
                    DateTime start =
                        DateTime.fromMillisecondsSinceEpoch(widget.startDate!);
                    DateTime end =
                        DateTime.fromMillisecondsSinceEpoch(widget.endDate!);
                    initialDateRange = DateTimeRange(start: start, end: end);
                  }

                  Future<void> selectDateRange() async {
                    final DateTimeRange? picked = await showDateRangePicker(
                      initialEntryMode: DatePickerEntryMode.calendar,
                      context: context,
                      firstDate: DateTime(DateTime.now().year - 2),
                      lastDate: DateTime.now(),
                      initialDateRange: initialDateRange,
                      builder: (BuildContext context, Widget? child) {
                        return Theme(
                          data: ThemeData.light().copyWith(
                            colorScheme: const ColorScheme.dark(
                              primary: Color.fromRGBO(49, 162, 215, 1),
                              onPrimary: Colors.black,
                              surface: Color.fromRGBO(49, 162, 215, 1),
                              onSurface: Color.fromRGBO(49, 162, 215, 1),
                            ),
                            dialogBackgroundColor:
                                const Color.fromRGBO(49, 162, 215, 1),
                          ),
                          child: child!,
                        );
                      },
                    );

                    if (picked == null) return;

                    if (picked != dateRange) {
                      dateRange = picked;
                      // log('Date range selected: ${dateRange!.start.toIso8601String().split('T')[0]} - ${dateRange!.end.toIso8601String().split('T')[0]}',
                      //     name: 'leaderboard app bar');
                    }

                    setState(() {
                      isDataFetched = true;
                    });

                    widget.refreshDataOnDateChanged!(
                      dateRange!.start.millisecondsSinceEpoch,
                      dateRange!.end.millisecondsSinceEpoch,
                      false,
                    );

                    // log('refreshing data, $isDataFetched',
                    //     name: 'leaderboard app bar');
                  }

                  selectDateRange();
                },
              )
            : const SizedBox(),
        widget.refreshDataOnDateChanged != null
            ? IconButton(
                icon: const Icon(Icons.refresh, color: Colors.white),
                onPressed: () {
                  DateTime now = DateTime.now();
                  DateTime firstDayOfMonth = DateTime(now.year, now.month, 1);

                  // log('refreshing data, $isDataFetched',
                  //     name: 'leaderboard app bar');

                  if (isDataFetched) {
                    widget.refreshDataOnDateChanged!(
                      firstDayOfMonth.millisecondsSinceEpoch,
                      now.millisecondsSinceEpoch,
                      true,
                    );

                    setState(() {
                      isDataFetched = false;
                    });
                  }
                },
              )
            : const SizedBox(),
      ],
      leading: IconButton(
        icon: const Icon(Icons.arrow_back_ios, color: Colors.white),
        onPressed: () {
          Navigator.of(context).pop();
        },
      ),
      title: const Text(
        'Leaderboard',
        style: TextStyle(
          color: Colors.white,
          fontSize: 16,
          fontWeight: FontWeight.normal,
        ),
      ),
    );
  }
}
