import 'dart:developer';

import 'package:nudge_leaderboard_ui/features/leaderboard/data/models/get_nudge_leaderboard.dart';
import 'package:nudge_leaderboard_ui/features/leaderboard/presentation/widgets/appbar/leaderboard_appbar.dart';
import 'package:nudge_leaderboard_ui/features/leaderboard/presentation/widgets/leaderboard_rankers_container.dart';
import 'package:nudge_leaderboard_ui/features/leaderboard/presentation/widgets/rank_container.dart';
import 'package:lazy_load_scrollview/lazy_load_scrollview.dart';
import 'package:flutter/material.dart';

import 'leaderboard_nodata.dart';

enum LeaderBoardType { defaultBased, timeBased }

class LeaderboardDataScore extends StatefulWidget {
  final GetNudgeLeaderboard? nudgeLeaderboardData;
  final GetNudgeLeaderboard? nudgeLeaderboardRankers;
  final int currentItemCount;
  final int currentPageNo;
  final Function(LeaderBoardType) loadMoreData;
  final Function(
    int,
    int,
    bool,
  ) refreshDataOnDateChanged;
  final int startDate;
  final int endDate;

  const LeaderboardDataScore({
    super.key,
    required this.nudgeLeaderboardData,
    required this.nudgeLeaderboardRankers,
    required this.currentItemCount,
    required this.currentPageNo,
    required this.loadMoreData,
    required this.refreshDataOnDateChanged,
    required this.startDate,
    required this.endDate,
  });

  @override
  State<LeaderboardDataScore> createState() => _LeaderboardDataScoreState();
}

class _LeaderboardDataScoreState extends State<LeaderboardDataScore> {
  bool rangeSelected = false;

  @override
  Widget build(BuildContext context) {
    int usersLength = widget.nudgeLeaderboardRankers?.data.users.length ?? 0;
    int usersLength2 = widget.nudgeLeaderboardData?.data.users.length ?? 0;
    User? you = widget.nudgeLeaderboardRankers?.data.user!;

    // log("rankers length: $usersLength\nusers length: $usersLength2",
    //     name: 'leaderboard scores');

    if (usersLength == 0) {
      return LeaderboardNoData();
    }

    return Scaffold(
      backgroundColor: const Color.fromRGBO(49, 162, 215, 1),
      appBar: LeaderboardAppBar(
        refreshDataOnDateChanged: widget.refreshDataOnDateChanged,
        startDate: widget.startDate,
        endDate: widget.endDate,
      ),
      body: (usersLength >= 5)
          ? Column(
              children: [
                const SizedBox(height: 10),
                Container(
                  width: double.infinity,
                  decoration: const BoxDecoration(
                    color: Color.fromRGBO(49, 162, 215, 1),
                    borderRadius: BorderRadius.only(
                      topLeft: Radius.circular(20),
                      topRight: Radius.circular(20),
                    ),
                  ),
                  child: Row(
                    children: [
                      LeaderboardRankers(
                        nameRank1: widget.nudgeLeaderboardRankers?.data.users[0]
                                .userName ??
                            'Loading...',
                        pointsRank1: widget
                                .nudgeLeaderboardRankers?.data.users[0].score ??
                            0,
                        nameRank2: widget.nudgeLeaderboardRankers?.data.users[1]
                                .userName ??
                            'Loading...',
                        pointsRank2: widget
                                .nudgeLeaderboardRankers?.data.users[1].score ??
                            0,
                        nameRank3: widget.nudgeLeaderboardRankers?.data.users[2]
                                .userName ??
                            'Loading...',
                        pointsRank3: widget
                                .nudgeLeaderboardRankers?.data.users[2].score ??
                            0,
                        photoUrl1: widget
                            .nudgeLeaderboardRankers?.data.users[0].photoUrl,
                        photoUrl2: widget
                            .nudgeLeaderboardRankers?.data.users[1].photoUrl,
                        photoUrl3: widget
                            .nudgeLeaderboardRankers?.data.users[2].photoUrl,
                        // photoUrl1: 'https://via.placeholder.com/150',
                        // photoUrl2: 'https://via.placeholder.com/150',
                        // photoUrl3: 'https://via.placeholder.com/150',
                      ),
                    ],
                  ),
                ),
                Container(
                  alignment: Alignment.centerLeft,
                  padding: const EdgeInsets.fromLTRB(
                    30,
                    10,
                    0,
                    10,
                  ),
                  child: const Text(
                    'You',
                    style: TextStyle(
                      fontSize: 18,
                      color: Colors.white,
                    ),
                  ),
                ),
                RankContainer(
                  points: you?.score ?? 0,
                  name: you?.userName ?? 'loading....',
                  rank: you?.rank ?? 0,
                  // photoUrl: nudgeLeaderboardData?.data.user?.userId?.photoUrl,
                ),
                const SizedBox(
                  height: 10,
                ),
                Container(
                  width: double.infinity,
                  decoration: const BoxDecoration(
                    color: Color.fromRGBO(246, 247, 250, 1),
                    borderRadius: BorderRadius.only(
                      topLeft: Radius.circular(20),
                      topRight: Radius.circular(20),
                    ),
                  ),
                  child: const Headers(),
                ),
                Flexible(
                  child: Container(
                    decoration: const BoxDecoration(
                      color: Color.fromRGBO(246, 247, 250, 1),
                    ),
                    child: widget.nudgeLeaderboardData!.data.users.isEmpty
                        ? const Padding(
                            padding: EdgeInsets.symmetric(
                              horizontal: 20.0,
                              vertical: 16.0,
                            ),
                            child: Center(
                              child: Text(
                                textAlign: TextAlign.center,
                                "No data for the specified date range",
                                style: TextStyle(
                                  fontSize: 20,
                                  fontWeight: FontWeight.bold,
                                  color: Colors.black,
                                ),
                              ),
                            ),
                          )
                        : LazyLoadScrollView(
                            // onEndOfPage: () => widget.loadMoreData(
                            //   DateTimeRange(
                            //     start: DateTime.fromMillisecondsSinceEpoch(
                            //         widget.startDate),
                            //     end: DateTime.fromMillisecondsSinceEpoch(
                            //         widget.endDate),
                            //   ),
                            // ),
                            onEndOfPage: () {
                              DateTime now = DateTime.now();
                              DateTime firstDayOfMonth =
                                  DateTime(now.year, now.month, 1);
                              DateTime startDateTime =
                                  DateTime.fromMillisecondsSinceEpoch(
                                      widget.startDate);
                              DateTime endDateTime =
                                  DateTime.fromMillisecondsSinceEpoch(
                                      widget.endDate);

                              // Create new DateTime objects that only include the year, month, and day
                              DateTime startDateTimeDateOnly = DateTime(
                                  startDateTime.year,
                                  startDateTime.month,
                                  startDateTime.day);
                              DateTime endDateTimeDateOnly = DateTime(
                                  endDateTime.year,
                                  endDateTime.month,
                                  endDateTime.day);
                              DateTime firstDayOfMonthDateOnly = DateTime(
                                  firstDayOfMonth.year,
                                  firstDayOfMonth.month,
                                  firstDayOfMonth.day);
                              DateTime nowDateOnly =
                                  DateTime(now.year, now.month, now.day);

                              if (startDateTimeDateOnly !=
                                      firstDayOfMonthDateOnly ||
                                  endDateTimeDateOnly != nowDateOnly) {
                                widget.loadMoreData(LeaderBoardType.timeBased);
                              } else {
                                widget
                                    .loadMoreData(LeaderBoardType.defaultBased);
                              }
                            },
                            child: SingleChildScrollView(
                              child: Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                children: [
                                  for (int index = 0;
                                      index < widget.currentItemCount &&
                                          index <
                                              (widget.nudgeLeaderboardData?.data
                                                      .users.length ??
                                                  0);
                                      index++)
                                    RankContainer(
                                      points: widget.nudgeLeaderboardData?.data
                                              .users[index].score ??
                                          0,
                                      name: widget.nudgeLeaderboardData?.data
                                              .users[index].userName ??
                                          'Loading...',
                                      rank: index + 1,
                                      // photoUrl: nudgeLeaderboardData
                                      //     ?.data.users[index].userId?.photoUrl,
                                    ),
                                ],
                              ),
                            ),
                          ),
                  ),
                ),
              ],
            )
          : const Center(
              child: Text(
                "Leaderboard would be enabled after there are at least 5 users",
                style: TextStyle(
                  fontSize: 20,
                  fontWeight: FontWeight.bold,
                  color: Colors.white,
                ),
              ),
            ),
    );
  }
}

class Headers extends StatelessWidget {
  const Headers({super.key});

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.only(
        left: 22.0,
        top: 16,
        right: 24,
        bottom: 8,
      ),
      child: Row(
        // mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Expanded(
            flex: 1,
            child: Container(
              padding: const EdgeInsets.only(left: 10.0),
              child: const Text(
                'Rank',
                style: TextStyle(
                  color: Colors.black,
                  fontSize: 16,
                  fontWeight: FontWeight.normal,
                ),
              ),
            ),
          ),
          Expanded(
            flex: 3,
            child: Container(
              padding: const EdgeInsets.only(left: 25.0),
              child: const Text(
                'Name',
                style: TextStyle(
                  color: Colors.black,
                  fontSize: 16,
                  fontWeight: FontWeight.normal,
                ),
              ),
            ),
          ),
          Expanded(
            flex: 1,
            child: Container(
              padding: const EdgeInsets.only(left: 10.0),
              width: MediaQuery.of(context).size.width * 0.15,
              child: const Text(
                'Points',
                style: TextStyle(
                  color: Colors.black,
                  fontSize: 16,
                  fontWeight: FontWeight.normal,
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }
}
